"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const cross_fetch_1 = __importStar(require("cross-fetch"));
const HttpMethod_js_1 = require("../types/HttpMethod.js");
class HttpClient {
    /**
     * Creates an instance of the {@link HttpClient} class.
     *
     * This is a bare bones abstraction around cross-fetch, meant to provide some simple interfaces for making HTTP requests.
     *
     * @constructor
     * @param userAgent The user agent to use for all requests
     */
    constructor(userAgent) {
        this.userAgent = userAgent;
    }
    /**
     * Make a HEAD request.
     *
     * Note: HEAD requests cannot contain bodies, and so any body provided in the RequestInit will be ignored.
     *
     * @param requestUrl The URL to make the request to
     * @param requestInit The settings for how the request should be made
     * @returns a {@link Response} object
     */
    head(requestUrl, requestInit = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const requestInitWithMethod = Object.assign(Object.assign({}, requestInit), { method: HttpMethod_js_1.HttpMethod.HEAD });
            return yield this.request(requestUrl, requestInitWithMethod);
        });
    }
    /**
     * Make an OPTIONS request.
     *
     * @param requestUrl The URL to make the request to
     * @param requestInit The settings for how the request should be made
     * @returns a {@link Response} object
     */
    options(requestUrl, requestInit = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const requestInitWithMethod = Object.assign(Object.assign({}, requestInit), { method: HttpMethod_js_1.HttpMethod.OPTIONS });
            return yield this.request(requestUrl, requestInitWithMethod);
        });
    }
    /**
     * Make a GET request.
     *
     * Note: GET requests cannot contain bodies, and so any body provided in the RequestInit will be ignored.
     *
     * @param requestUrl The URL to make the request to
     * @param requestInit The settings for how the request should be made
     * @returns a {@link Response} object
     */
    get(requestUrl, requestInit = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const requestInitWithMethod = Object.assign(Object.assign({}, requestInit), { method: HttpMethod_js_1.HttpMethod.GET });
            return yield this.request(requestUrl, requestInitWithMethod);
        });
    }
    /**
     * Make a DELETE request.
     *
     * @param requestUrl The URL to make the request to
     * @param requestInit The settings for how the request should be made
     * @returns a {@link Response} object
     */
    delete(requestUrl, requestInit = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const requestInitWithMethod = Object.assign(Object.assign({}, requestInit), { method: HttpMethod_js_1.HttpMethod.DELETE });
            return yield this.request(requestUrl, requestInitWithMethod);
        });
    }
    /**
     * Make a POST request.
     *
     * @param requestUrl The URL to make the request to
     * @param requestInit The settings for how the request should be made
     * @returns a {@link Response} object
     */
    post(requestUrl, requestInit = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const requestInitWithMethod = Object.assign(Object.assign({}, requestInit), { method: HttpMethod_js_1.HttpMethod.POST });
            return yield this.request(requestUrl, requestInitWithMethod);
        });
    }
    /**
     * Make a PATCH request.
     *
     * @param requestUrl The URL to make the request to
     * @param requestInit The settings for how the request should be made
     * @returns a {@link Response} object
     */
    patch(requestUrl, requestInit = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const requestInitWithMethod = Object.assign(Object.assign({}, requestInit), { method: HttpMethod_js_1.HttpMethod.PATCH });
            return yield this.request(requestUrl, requestInitWithMethod);
        });
    }
    /**
     * Make a PUT request.
     *
     * @param requestUrl The URL to make the request to
     * @param requestInit The settings for how the request should be made
     * @returns a {@link Response} object
     */
    put(requestUrl, requestInit = {}) {
        return __awaiter(this, void 0, void 0, function* () {
            const requestInitWithMethod = Object.assign(Object.assign({}, requestInit), { method: HttpMethod_js_1.HttpMethod.PUT });
            return yield this.request(requestUrl, requestInitWithMethod);
        });
    }
    /**
     * Makes a raw fetch request.
     * All other methods such as get, post, patch, and request ultimately call this.
     * The normal get, post, patch, et all methods should be preferred.
     *
     * @param requestUrl The URL to make the request to
     * @param requestInit The {@link MethodSpecificRequestInit} settings for how the request should be made
     * @returns a {@link Response} object
     */
    request(requestUrl, requestInit) {
        return __awaiter(this, void 0, void 0, function* () {
            // make copy of requestInit to make sure the original is maintained
            const copyOfRequestInit = Object.assign({}, requestInit);
            copyOfRequestInit.headers = this._getHeadersWithUserAgentMixedIn(copyOfRequestInit.headers);
            // Form the Request object.
            const request = new cross_fetch_1.Request(requestUrl, copyOfRequestInit);
            // Use the Request object to make the request.
            const response = yield (0, cross_fetch_1.default)(request);
            return response;
        });
    }
    /**
     * Create the {@link Headers} object to be used for the request.
     *
     * A {@link Headers} object is preferred by cross-fetch for the headers. It also allows for case-insensitive
     * lookups of header keys, which is convenient.
     *
     * @param headers The {@link HeadersInit} settings that will be used to produce the headers for the request
     * @returns The formed {@link Headers} object
     */
    _getHeadersWithUserAgentMixedIn(headers) {
        const userAgentUpdatedHeaders = new cross_fetch_1.Headers(headers);
        if (this.userAgent) {
            userAgentUpdatedHeaders.set("User-Agent", this.userAgent);
        }
        return userAgentUpdatedHeaders;
    }
}
exports.default = HttpClient;
//# sourceMappingURL=HttpClient.js.map