"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chai_1 = require("chai");
const chai_as_promised_1 = __importDefault(require("chai-as-promised"));
const nock_1 = __importDefault(require("nock"));
const HttpClient_js_1 = __importDefault(require("./HttpClient.js"));
(0, chai_1.use)(chai_as_promised_1.default);
describe("HttpClient", function () {
    let httpClient;
    let nockScope;
    beforeEach(function () {
        httpClient = new HttpClient_js_1.default("typed-test-client-tests");
    });
    afterEach(function () {
        nock_1.default.cleanAll();
    });
    describe("HTTP", function () {
        let rootUrl;
        beforeEach(function () {
            rootUrl = "http://localhost:80";
            nockScope = (0, nock_1.default)(rootUrl);
        });
        describe("GET", function () {
            describe("Basic", function () {
                beforeEach(function () {
                    nockScope
                        .get("/get")
                        .reply(200, function (uri, _reqBody) {
                        const context = this;
                        return {
                            method: context.req.method,
                            headers: context.req.headers,
                            url: uri,
                        };
                    });
                });
                it("succeeds", function () {
                    return __awaiter(this, void 0, void 0, function* () {
                        const res = yield httpClient.get(new URL("http://localhost:80/get"));
                        (0, chai_1.expect)(res.status).to.equal(200);
                        const body = yield res.text();
                        const obj = JSON.parse(body);
                        (0, chai_1.expect)(Object.keys(obj.headers)).to.include("user-agent");
                    });
                });
                it("succeeds with undefined agent", function () {
                    return __awaiter(this, void 0, void 0, function* () {
                        const http = new HttpClient_js_1.default(undefined);
                        const res = yield http.get(new URL("http://localhost:80/get"));
                        (0, chai_1.expect)(res.status).to.equal(200);
                        const body = yield res.text();
                        const obj = JSON.parse(body);
                        (0, chai_1.expect)(Object.keys(obj.headers)).to.include("user-agent");
                        (0, chai_1.expect)(obj.headers["user-agent"]).to.deep.equal([
                            "node-fetch/1.0 (+https://github.com/bitinn/node-fetch)",
                        ]);
                    });
                });
                it("succeeds with empty agent", function () {
                    return __awaiter(this, void 0, void 0, function* () {
                        const http = new HttpClient_js_1.default("");
                        const res = yield http.get(new URL("http://localhost:80/get"));
                        (0, chai_1.expect)(res.status).to.equal(200);
                        const body = yield res.text();
                        const obj = JSON.parse(body);
                        (0, chai_1.expect)(Object.keys(obj.headers)).to.include("user-agent");
                        (0, chai_1.expect)(obj.headers["user-agent"]).to.deep.equal([
                            "node-fetch/1.0 (+https://github.com/bitinn/node-fetch)",
                        ]);
                    });
                });
            });
            describe("No charset in response", function () {
                beforeEach(function () {
                    nockScope.get("/get").reply(200, function (uri, _reqBody) {
                        const context = this;
                        return {
                            method: context.req.method,
                            headers: context.req.headers,
                            url: uri,
                        };
                    }, { "content-type": "application/json" });
                });
                it("succeeds", function () {
                    return __awaiter(this, void 0, void 0, function* () {
                        const res = yield httpClient.get(new URL("http://localhost:80/get"));
                        (0, chai_1.expect)(res.status).to.equal(200);
                        yield (0, chai_1.expect)(res.text()).to.eventually.be.fulfilled;
                    });
                });
            });
            describe("Compatible charset in response", function () {
                beforeEach(function () {
                    nockScope.get("/get").reply(200, function (uri, _reqBody) {
                        const context = this;
                        return {
                            method: context.req.method,
                            headers: context.req.headers,
                            url: uri,
                        };
                    }, { "content-type": "application/json; charset=utf8" });
                });
                it("succeeds", function () {
                    return __awaiter(this, void 0, void 0, function* () {
                        const res = yield httpClient.get(new URL("http://localhost:80/get"));
                        (0, chai_1.expect)(res.status).to.equal(200);
                        yield (0, chai_1.expect)(res.text()).to.eventually.be.fulfilled;
                    });
                });
            });
            describe("Incompatible charset in response", function () {
                beforeEach(function () {
                    nockScope.get("/get").reply(200, function (uri, _reqBody) {
                        const context = this;
                        return {
                            method: context.req.method,
                            headers: context.req.headers,
                            url: uri,
                        };
                    }, { "content-type": "application/json; charset=utf-9" });
                });
                it("succeeds", function () {
                    return __awaiter(this, void 0, void 0, function* () {
                        const res = yield httpClient.get(new URL("http://localhost:80/get"));
                        (0, chai_1.expect)(res.status).to.equal(200);
                        yield (0, chai_1.expect)(res.text()).to.eventually.be.fulfilled;
                    });
                });
            });
            describe("404", function () {
                beforeEach(function () {
                    nockScope
                        .get("/status/404")
                        .reply(404, function (uri, _reqBody) {
                        const context = this;
                        return {
                            method: context.req.method,
                            headers: context.req.headers,
                            url: uri,
                        };
                    });
                });
                it("succeeds", function () {
                    return __awaiter(this, void 0, void 0, function* () {
                        const res = yield httpClient.get(new URL("http://localhost:80/status/404"));
                        (0, chai_1.expect)(res.status).to.equal(404);
                        yield (0, chai_1.expect)(res.text()).to.eventually.be.fulfilled;
                    });
                });
            });
        });
        describe("HEAD", function () {
            beforeEach(function () {
                nockScope
                    .head("/head")
                    .reply(200, function (uri, _reqBody) {
                    const context = this;
                    return {
                        method: context.req.method,
                        headers: context.req.headers,
                        url: uri,
                    };
                });
            });
            it("succeeds", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const res = yield httpClient.head(new URL("http://localhost:80/head"));
                    (0, chai_1.expect)(res.status).to.equal(200);
                });
            });
        });
        describe("POST", function () {
            beforeEach(function () {
                nockScope
                    .post("/post")
                    .reply(200, function (_uri, reqBody) {
                    return {
                        data: reqBody,
                    };
                });
            });
            it("succeeds", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const data = "Hello World!";
                    const res = yield httpClient.post(new URL("http://localhost:80/post"), { body: data });
                    (0, chai_1.expect)(res.status).to.equal(200);
                    const body = yield res.text();
                    const obj = JSON.parse(body);
                    (0, chai_1.expect)(obj.data).to.equal(data);
                });
            });
            it("succeeds without body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const res = yield httpClient.post(new URL("http://localhost:80/post"));
                    (0, chai_1.expect)(res.status).to.equal(200);
                    const body = yield res.text();
                    const obj = JSON.parse(body);
                    (0, chai_1.expect)(obj.data).to.equal("");
                });
            });
        });
        describe("PATCH", function () {
            beforeEach(function () {
                nockScope
                    .patch("/patch")
                    .reply(200, function (_uri, reqBody) {
                    return {
                        data: reqBody,
                    };
                });
            });
            it("succeeds", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const data = "Hello World!";
                    const res = yield httpClient.patch(new URL("http://localhost:80/patch"), { body: data });
                    (0, chai_1.expect)(res.status).to.equal(200);
                    const body = yield res.text();
                    const obj = JSON.parse(body);
                    (0, chai_1.expect)(obj.data).to.equal(data);
                });
            });
            it("succeeds without body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const res = yield httpClient.patch(new URL("http://localhost:80/patch"));
                    (0, chai_1.expect)(res.status).to.equal(200);
                    const body = yield res.text();
                    const obj = JSON.parse(body);
                    (0, chai_1.expect)(obj.data).to.equal("");
                });
            });
        });
        describe("PUT", function () {
            beforeEach(function () {
                nockScope
                    .put("/put")
                    .reply(200, function (_uri, reqBody) {
                    return {
                        data: reqBody,
                    };
                });
            });
            it("succeeds", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const data = "Hello World!";
                    const res = yield httpClient.put(new URL("http://localhost:80/put"), { body: data });
                    (0, chai_1.expect)(res.status).to.equal(200);
                    const body = yield res.text();
                    const obj = JSON.parse(body);
                    (0, chai_1.expect)(obj.data).to.equal(data);
                });
            });
            it("succeeds without body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const res = yield httpClient.put(new URL("http://localhost:80/put"));
                    (0, chai_1.expect)(res.status).to.equal(200);
                    const body = yield res.text();
                    const obj = JSON.parse(body);
                    (0, chai_1.expect)(obj.data).to.equal("");
                });
            });
        });
        describe("DELETE", function () {
            beforeEach(function () {
                nockScope
                    .delete("/delete")
                    .reply(200, function (uri, _reqBody) {
                    const context = this;
                    return {
                        method: context.req.method,
                        headers: context.req.headers,
                        url: uri,
                    };
                });
            });
            it("succeeds", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const res = yield httpClient.delete(new URL("http://localhost:80/delete"));
                    (0, chai_1.expect)(res.status).to.equal(200);
                    const body = yield res.text();
                    (0, chai_1.expect)(typeof JSON.parse(body)).to.equal("object");
                });
            });
        });
        describe("OPTIONS", function () {
            beforeEach(function () {
                nockScope
                    .options("/options")
                    .reply(200, function (uri, _reqBody) {
                    const context = this;
                    return {
                        method: context.req.method,
                        headers: context.req.headers,
                        url: uri,
                    };
                });
            });
            it("succeeds", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const res = yield httpClient.options(new URL("http://localhost:80/options"));
                    (0, chai_1.expect)(res.status).to.equal(200);
                    yield (0, chai_1.expect)(res.text()).to.eventually.be.fulfilled;
                });
            });
        });
    });
    describe("HTTPS", function () {
        let rootUrl;
        beforeEach(function () {
            rootUrl = "https://localhost:80";
            nockScope = (0, nock_1.default)(rootUrl);
        });
        describe("GET", function () {
            describe("Basic", function () {
                beforeEach(function () {
                    nockScope
                        .get("/get")
                        .reply(200, function (uri, _reqBody) {
                        const context = this;
                        return {
                            method: context.req.method,
                            headers: context.req.headers,
                            url: uri,
                        };
                    });
                });
                it("succeeds", function () {
                    return __awaiter(this, void 0, void 0, function* () {
                        const res = yield httpClient.get(new URL("https://localhost:80/get"));
                        (0, chai_1.expect)(res.status).to.equal(200);
                        const body = yield res.text();
                        const obj = JSON.parse(body);
                        (0, chai_1.expect)(Object.keys(obj.headers)).to.include("user-agent");
                    });
                });
            });
        });
    });
});
//# sourceMappingURL=HttpClient.spec.js.map