"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const tslib_1 = require("tslib");
const chai_1 = require("chai");
const chai_as_promised_1 = __importDefault(require("chai-as-promised"));
const cross_fetch_1 = require("cross-fetch");
const nock_1 = __importDefault(require("nock"));
const Object_1 = require("primitive-predicates/Object");
const Property_1 = require("primitive-predicates/Property");
const String_1 = require("primitive-predicates/String");
const index_js_1 = require("../contentTypeHandlers/index.js");
const errors_js_1 = require("../errors.js");
const Date_js_1 = require("../JsonRevivers/Date.js");
const index_js_2 = __importDefault(require("../index.js"));
(0, chai_1.use)(chai_as_promised_1.default);
function isRawResponseData(value) {
    return (!!value &&
        (0, Object_1.isObject)(value) &&
        (0, Property_1.hasProperty)(value, "status") &&
        (0, String_1.isString)(value.status));
}
function parseRawResponseData({ responseBodyAsObject, }) {
    if (isRawResponseData(responseBodyAsObject)) {
        return Object.assign({}, responseBodyAsObject);
    }
    throw new TypeError(`Response body is not RawResponseData: ${JSON.stringify(responseBodyAsObject)}`);
}
const standardUrl = new URL("https://localhost:80");
const payload = {
    something: "hello",
    other: "world",
};
const urlWithQueryString = new URL("https://localhost:80");
for (const [key, value] of Object.entries(payload)) {
    urlWithQueryString.searchParams.set(key, value);
}
describe("TypedHttpClient", function () {
    describe("HEAD requests", function () {
        let client;
        let requestBody;
        let requestMethod;
        let requestHeaders;
        let restRes;
        describe("With query string", function () {
            let requestUri;
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .head(new RegExp("/.*"))
                        .reply(200, function (uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        requestUri = uri;
                        return { status: "ok" };
                    });
                    restRes = yield client.head({
                        url: urlWithQueryString,
                    });
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal(undefined);
            });
            it("uses null header because it has no body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal(null);
                });
            });
            it("request body is empty", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestBody).to.deep.equal("");
                });
            });
            it("uses HEAD", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("HEAD");
                });
            });
            it("uses query string", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const params = new URL(`https://localhost:80${requestUri}`)
                        .searchParams;
                    (0, chai_1.expect)(params.get("something")).to.equal("hello");
                    (0, chai_1.expect)(params.get("other")).to.equal("world");
                });
            });
        });
        describe("Without body", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .head("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.head({ url: standardUrl });
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal(undefined);
            });
            it("uses null header because it has no body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal(null);
                });
            });
            it("used empty string for request body", function () {
                (0, chai_1.expect)(requestBody).to.equal("");
            });
            it("uses HEAD", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("HEAD");
                });
            });
        });
        describe("Without body and with options", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .head("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.head({
                        url: standardUrl,
                        additionalHeaders: { "extra-header": "3" },
                    });
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal(undefined);
            });
            it("uses null header because it has no body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal(null);
                });
            });
            it("uses extra header", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("extra-header")).to.equal("3");
                });
            });
            it("used empty string for request body", function () {
                (0, chai_1.expect)(requestBody).to.equal("");
            });
            it("uses HEAD", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("HEAD");
                });
            });
        });
    });
    describe("GET requests", function () {
        let client;
        let requestBody;
        let requestMethod;
        let requestHeaders;
        let restRes;
        describe("With query string", function () {
            let requestUri;
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .get(new RegExp("/.*"))
                        .reply(200, function (uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        requestUri = uri;
                        return { status: "ok" };
                    }, {
                        "content-type": "application/json",
                    });
                    restRes = yield client.get({ url: urlWithQueryString }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses null header because it has no body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal(null);
                });
            });
            it("request body is empty", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestBody).to.deep.equal("");
                });
            });
            it("uses GET", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("GET");
                });
            });
            it("uses query string", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const params = new URL(`https://localhost:80${requestUri}`)
                        .searchParams;
                    (0, chai_1.expect)(params.get("something")).to.equal("hello");
                    (0, chai_1.expect)(params.get("other")).to.equal("world");
                });
            });
        });
        describe("With query string and response with charset in Content-Type header", function () {
            let requestUri;
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .get(new RegExp("/.*"))
                        .reply(200, function (uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        requestUri = uri;
                        return { status: "ok" };
                    }, {
                        "content-type": "application/json; charset=utf-8",
                    });
                    restRes = yield client.get({ url: urlWithQueryString }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses null header because it has no body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal(null);
                });
            });
            it("request body is empty", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestBody).to.deep.equal("");
                });
            });
            it("uses GET", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("GET");
                });
            });
            it("uses query string", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    const params = new URL(`https://localhost:80${requestUri}`)
                        .searchParams;
                    (0, chai_1.expect)(params.get("something")).to.equal("hello");
                    (0, chai_1.expect)(params.get("other")).to.equal("world");
                });
            });
        });
        describe("With Reviver", function () {
            const expectedDate = new Date();
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .get("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok", myDate: expectedDate.toISOString() };
                    });
                    restRes = yield client.get({
                        url: standardUrl,
                        responseJsonReviver: Date_js_1.JsonISO8601DateAndTimeReviver,
                    }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({
                    status: "ok",
                    myDate: expectedDate,
                });
            });
            it("uses null header because it has no body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal(null);
                });
            });
            it("used empty string for request body", function () {
                (0, chai_1.expect)(requestBody).to.equal("");
            });
            it("uses GET", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("GET");
                });
            });
        });
        describe("Without body", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .get("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.get({ url: standardUrl }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses null header because it has no body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal(null);
                });
            });
            it("used empty string for request body", function () {
                (0, chai_1.expect)(requestBody).to.equal("");
            });
            it("uses GET", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("GET");
                });
            });
        });
        describe("Without body and with options", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .get("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.get({ url: standardUrl, additionalHeaders: { "extra-header": "3" } }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses null header because it has no body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal(null);
                });
            });
            it("uses extra header", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("extra-header")).to.equal("3");
                });
            });
            it("used empty string for request body", function () {
                (0, chai_1.expect)(requestBody).to.equal("");
            });
            it("uses GET", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("GET");
                });
            });
        });
        describe("Empty string for response with text content type", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80").get("/").reply(200, "");
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("throws TypeError", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    yield (0, chai_1.expect)(client.get({ url: standardUrl }, parseRawResponseData)).to.eventually.be.rejectedWith(TypeError);
                });
            });
        });
        describe("Empty string for response with JSON content type", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .get("/")
                        .reply(200, "", { "content-type": "application/json" });
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("throws TypeError", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    yield (0, chai_1.expect)(client.get({ url: standardUrl }, parseRawResponseData)).to.eventually.be.rejectedWith(TypeError);
                });
            });
        });
        describe("With improper JSON response", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .get("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return "{";
                    }, { "content-type": "application/json" });
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("throws ResponseBodyNotJSONError", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    yield (0, chai_1.expect)(client.get({ url: standardUrl }, parseRawResponseData)).to.eventually.be.rejectedWith(errors_js_1.ResponseBodyNotJSONError);
                });
            });
        });
    });
    describe("POST requests", function () {
        let client;
        let requestBody;
        let requestMethod;
        let requestHeaders;
        let restRes;
        describe("With body (generics provided)", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .post("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.post({ url: standardUrl, payload }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses application/json header by default", function () {
                (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal("application/json; charset=utf8");
            });
            it("encoded request body in JSON format by default", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestBody).to.deep.equal(payload);
                });
            });
            it("uses POST", function () {
                (0, chai_1.expect)(requestMethod).to.equal("POST");
            });
        });
        describe("With body (generics inferred)", function () {
            const payload = {
                something: "hello",
                other: "world",
            };
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .post("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.post({ url: standardUrl, payload }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses application/json header by default", function () {
                (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal("application/json; charset=utf8");
            });
            it("encoded request body in JSON format by default", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestBody).to.deep.equal(payload);
                });
            });
            it("uses POST", function () {
                (0, chai_1.expect)(requestMethod).to.equal("POST");
            });
        });
        describe("With body and options", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .post("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.post({
                        url: standardUrl,
                        payload,
                        additionalHeaders: { "extra-header": "3" },
                        contentTypeHandler: index_js_1.WWWFormEncodedContentTypeHandler,
                    }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses x-www-form-urlencoded header", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal("application/x-www-form-urlencoded; charset=utf8");
                });
            });
            it("uses extra header", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("extra-header")).to.equal("3");
                });
            });
            it("encoded request body in x-www-form-urlencoded format", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestBody).to.equal("something=hello&other=world");
                });
            });
            it("uses POST", function () {
                (0, chai_1.expect)(requestMethod).to.equal("POST");
            });
        });
        describe("Without body (generic provided)", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .post("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    }, { "content-type": "application/json" });
                    restRes = yield client.post({ url: standardUrl }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses null header because it has no body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal(null);
                });
            });
            it("used empty string for request body", function () {
                (0, chai_1.expect)(requestBody).to.equal("");
            });
            it("uses POST", function () {
                (0, chai_1.expect)(requestMethod).to.equal("POST");
            });
        });
        describe("Without body (generic inferred)", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .post("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    }, { "content-type": "application/json" });
                    restRes = yield client.post({ url: standardUrl }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses null header because it has no body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal(null);
                });
            });
            it("used empty string for request body", function () {
                (0, chai_1.expect)(requestBody).to.equal("");
            });
            it("uses POST", function () {
                (0, chai_1.expect)(requestMethod).to.equal("POST");
            });
        });
        describe("Without body and with options", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .post("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.post({ url: standardUrl, additionalHeaders: { "extra-header": "3" } }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses null header because it has no body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal(null);
                });
            });
            it("uses extra header", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("extra-header")).to.equal("3");
                });
            });
            it("used empty string for request body", function () {
                (0, chai_1.expect)(requestBody).to.equal("");
            });
            it("uses POST", function () {
                (0, chai_1.expect)(requestMethod).to.equal("POST");
            });
        });
    });
    describe("PUT requests", function () {
        let client;
        let requestBody;
        let requestMethod;
        let requestHeaders;
        let restRes;
        describe("With body (generics provided)", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .put("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.put({ url: standardUrl, payload }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses application/json header by default", function () {
                (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal("application/json; charset=utf8");
            });
            it("encoded request body in JSON format by default", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestBody).to.deep.equal(payload);
                });
            });
            it("uses PUT", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("PUT");
                });
            });
        });
        describe("With body (generics inferred)", function () {
            const payload = {
                something: "hello",
                other: "world",
            };
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .put("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.put({ url: standardUrl, payload }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses application/json header by default", function () {
                (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal("application/json; charset=utf8");
            });
            it("encoded request body in JSON format by default", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestBody).to.deep.equal(payload);
                });
            });
            it("uses PUT", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("PUT");
                });
            });
        });
        describe("With body and options", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .put("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.put({
                        url: standardUrl,
                        payload,
                        additionalHeaders: { "extra-header": "3" },
                        contentTypeHandler: index_js_1.WWWFormEncodedContentTypeHandler,
                    }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses x-www-form-urlencoded header", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal("application/x-www-form-urlencoded; charset=utf8");
                });
            });
            it("uses extra header", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("extra-header")).to.equal("3");
                });
            });
            it("encoded request body in x-www-form-urlencoded format", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestBody).to.equal("something=hello&other=world");
                });
            });
            it("uses PUT", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("PUT");
                });
            });
        });
        describe("Without body (generic provided)", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .put("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.put({ url: standardUrl }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses null header because it has no body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal(null);
                });
            });
            it("used empty string for request body", function () {
                (0, chai_1.expect)(requestBody).to.equal("");
            });
            it("uses PUT", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("PUT");
                });
            });
        });
        describe("Without body (generic inferred)", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .put("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.put({ url: standardUrl }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses null header because it has no body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal(null);
                });
            });
            it("used empty string for request body", function () {
                (0, chai_1.expect)(requestBody).to.equal("");
            });
            it("uses PUT", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("PUT");
                });
            });
        });
        describe("Without body and with options", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .put("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.put({ url: standardUrl, additionalHeaders: { "extra-header": "3" } }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses null header because it has no body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal(null);
                });
            });
            it("uses extra header", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("extra-header")).to.equal("3");
                });
            });
            it("used empty string for request body", function () {
                (0, chai_1.expect)(requestBody).to.equal("");
            });
            it("uses PUT", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("PUT");
                });
            });
        });
    });
    describe("PATCH requests", function () {
        let client;
        let requestBody;
        let requestMethod;
        let requestHeaders;
        let restRes;
        describe("With body (generics provided)", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .patch("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.patch({ url: standardUrl, payload }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses application/json header by default", function () {
                (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal("application/json; charset=utf8");
            });
            it("encoded request body in JSON format by default", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestBody).to.deep.equal(payload);
                });
            });
            it("uses PATCH", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("PATCH");
                });
            });
        });
        describe("With body (generics inferred)", function () {
            const payload = {
                something: "hello",
                other: "world",
            };
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .patch("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.patch({ url: standardUrl, payload }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses application/json header by default", function () {
                (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal("application/json; charset=utf8");
            });
            it("encoded request body in JSON format by default", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestBody).to.deep.equal(payload);
                });
            });
            it("uses PATCH", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("PATCH");
                });
            });
        });
        describe("With body and options", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .patch("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.patch({
                        url: standardUrl,
                        payload,
                        additionalHeaders: { "extra-header": "3" },
                        contentTypeHandler: index_js_1.WWWFormEncodedContentTypeHandler,
                    }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses x-www-form-urlencoded header", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal("application/x-www-form-urlencoded; charset=utf8");
                });
            });
            it("uses extra header", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("extra-header")).to.equal("3");
                });
            });
            it("encoded request body in x-www-form-urlencoded format", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestBody).to.equal("something=hello&other=world");
                });
            });
            it("uses PATCH", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("PATCH");
                });
            });
        });
        describe("Without body (generic provided)", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .patch("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.patch({ url: standardUrl }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses null header because it has no body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal(null);
                });
            });
            it("used empty string for request body", function () {
                (0, chai_1.expect)(requestBody).to.equal("");
            });
            it("uses PATCH", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("PATCH");
                });
            });
        });
        describe("Without body (generic inferred)", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .patch("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.patch({ url: standardUrl }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses null header because it has no body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal(null);
                });
            });
            it("used empty string for request body", function () {
                (0, chai_1.expect)(requestBody).to.equal("");
            });
            it("uses PATCH", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("PATCH");
                });
            });
        });
        describe("Without body and with options", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .patch("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.patch({ url: standardUrl, additionalHeaders: { "extra-header": "3" } }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses null header because it has no body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal(null);
                });
            });
            it("uses extra header", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("extra-header")).to.equal("3");
                });
            });
            it("used empty string for request body", function () {
                (0, chai_1.expect)(requestBody).to.equal("");
            });
            it("uses PATCH", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("PATCH");
                });
            });
        });
    });
    describe("OPTIONS requests", function () {
        let client;
        let requestBody;
        let requestMethod;
        let requestHeaders;
        let restRes;
        describe("With body (generics provided)", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .options("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.options({ url: standardUrl, payload }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses application/json header by default", function () {
                (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal("application/json; charset=utf8");
            });
            it("encoded request body in JSON format by default", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestBody).to.deep.equal(payload);
                });
            });
            it("uses OPTIONS", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("OPTIONS");
                });
            });
        });
        describe("With body (generics inferred)", function () {
            const payload = {
                something: "hello",
                other: "world",
            };
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .options("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.options({ url: standardUrl, payload }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses application/json header by default", function () {
                (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal("application/json; charset=utf8");
            });
            it("encoded request body in JSON format by default", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestBody).to.deep.equal(payload);
                });
            });
            it("uses OPTIONS", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("OPTIONS");
                });
            });
        });
        describe("With body and options", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .options("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.options({
                        url: standardUrl,
                        payload,
                        additionalHeaders: { "extra-header": "3" },
                        contentTypeHandler: index_js_1.WWWFormEncodedContentTypeHandler,
                    }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses x-www-form-urlencoded header", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal("application/x-www-form-urlencoded; charset=utf8");
                });
            });
            it("uses extra header", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("extra-header")).to.equal("3");
                });
            });
            it("encoded request body in x-www-form-urlencoded format", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestBody).to.equal("something=hello&other=world");
                });
            });
            it("uses OPTIONS", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("OPTIONS");
                });
            });
        });
        describe("Without body (generic provided)", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .options("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.options({ url: standardUrl }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses null header because it has no body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal(null);
                });
            });
            it("used empty string for request body", function () {
                (0, chai_1.expect)(requestBody).to.equal("");
            });
            it("uses OPTIONS", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("OPTIONS");
                });
            });
        });
        describe("Without body (generic inferred)", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .options("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.options({ url: standardUrl }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses null header because it has no body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal(null);
                });
            });
            it("used empty string for request body", function () {
                (0, chai_1.expect)(requestBody).to.equal("");
            });
            it("uses OPTIONS", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("OPTIONS");
                });
            });
        });
        describe("Without body and with options", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .options("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.options({ url: standardUrl, additionalHeaders: { "extra-header": "3" } }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses null header because it has no body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal(null);
                });
            });
            it("uses extra header", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("extra-header")).to.equal("3");
                });
            });
            it("used empty string for request body", function () {
                (0, chai_1.expect)(requestBody).to.equal("");
            });
            it("uses OPTIONS", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("OPTIONS");
                });
            });
        });
    });
    describe("DELETE requests", function () {
        let client;
        let requestBody;
        let requestMethod;
        let requestHeaders;
        let restRes;
        describe("With body (generics provided)", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .delete("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.delete({ url: standardUrl, payload }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses application/json header by default", function () {
                (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal("application/json; charset=utf8");
            });
            it("encoded request body in JSON format by default", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestBody).to.deep.equal(payload);
                });
            });
            it("uses DELETE", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("DELETE");
                });
            });
        });
        describe("With body (generics inferred)", function () {
            const payload = {
                something: "hello",
                other: "world",
            };
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .delete("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.delete({ url: standardUrl, payload }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses application/json header by default", function () {
                (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal("application/json; charset=utf8");
            });
            it("encoded request body in JSON format by default", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestBody).to.deep.equal(payload);
                });
            });
            it("uses DELETE", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("DELETE");
                });
            });
        });
        describe("With body and options", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .delete("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.delete({
                        url: standardUrl,
                        payload,
                        additionalHeaders: { "extra-header": "3" },
                        contentTypeHandler: index_js_1.WWWFormEncodedContentTypeHandler,
                    }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses x-www-form-urlencoded header", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal("application/x-www-form-urlencoded; charset=utf8");
                });
            });
            it("uses extra header", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("extra-header")).to.equal("3");
                });
            });
            it("encoded request body in x-www-form-urlencoded format", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestBody).to.equal("something=hello&other=world");
                });
            });
            it("uses DELETE", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("DELETE");
                });
            });
        });
        describe("Without body (generic provided)", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .delete("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.delete({ url: standardUrl }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses null header because it has no body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal(null);
                });
            });
            it("used empty string for request body", function () {
                (0, chai_1.expect)(requestBody).to.equal("");
            });
            it("uses DELETE", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("DELETE");
                });
            });
        });
        describe("Without body (generic inferred)", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .delete("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.delete({ url: standardUrl }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses null header because it has no body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal(null);
                });
            });
            it("used empty string for request body", function () {
                (0, chai_1.expect)(requestBody).to.equal("");
            });
            it("uses DELETE", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("DELETE");
                });
            });
        });
        describe("Without body and with options", function () {
            before(function () {
                return __awaiter(this, void 0, void 0, function* () {
                    client = new index_js_2.default("typed-http-client-tests");
                    (0, nock_1.default)("https://localhost:80")
                        .delete("/")
                        .reply(200, function (_uri, reqBody) {
                        requestHeaders = new cross_fetch_1.Headers(this.req.headers);
                        requestMethod = this.req.method;
                        requestBody = reqBody;
                        return { status: "ok" };
                    });
                    restRes = yield client.delete({ url: standardUrl, additionalHeaders: { "extra-header": "3" } }, parseRawResponseData);
                });
            });
            after(function () {
                nock_1.default.cleanAll();
            });
            it("received response", function () {
                (0, chai_1.expect)(restRes.result).to.deep.equal({ status: "ok" });
            });
            it("uses null header because it has no body", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("content-type")).to.equal(null);
                });
            });
            it("uses extra header", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestHeaders.get("extra-header")).to.equal("3");
                });
            });
            it("used empty string for request body", function () {
                (0, chai_1.expect)(requestBody).to.equal("");
            });
            it("uses DELETE", function () {
                return __awaiter(this, void 0, void 0, function* () {
                    (0, chai_1.expect)(requestMethod).to.equal("DELETE");
                });
            });
        });
    });
});
//# sourceMappingURL=TypedHttpClient.spec.js.map