import fetch, { Headers, Request } from "cross-fetch";
import { HttpMethod } from "../types/HttpMethod.js";
export default class HttpClient {
    userAgent;
    /**
     * Creates an instance of the {@link HttpClient} class.
     *
     * This is a bare bones abstraction around cross-fetch, meant to provide some simple interfaces for making HTTP requests.
     *
     * @constructor
     * @param userAgent The user agent to use for all requests
     */
    constructor(userAgent) {
        this.userAgent = userAgent;
    }
    /**
     * Make a HEAD request.
     *
     * Note: HEAD requests cannot contain bodies, and so any body provided in the RequestInit will be ignored.
     *
     * @param requestUrl The URL to make the request to
     * @param requestInit The settings for how the request should be made
     * @returns a {@link Response} object
     */
    async head(requestUrl, requestInit = {}) {
        const requestInitWithMethod = {
            ...requestInit,
            method: HttpMethod.HEAD,
        };
        return await this.request(requestUrl, requestInitWithMethod);
    }
    /**
     * Make an OPTIONS request.
     *
     * @param requestUrl The URL to make the request to
     * @param requestInit The settings for how the request should be made
     * @returns a {@link Response} object
     */
    async options(requestUrl, requestInit = {}) {
        const requestInitWithMethod = {
            ...requestInit,
            method: HttpMethod.OPTIONS,
        };
        return await this.request(requestUrl, requestInitWithMethod);
    }
    /**
     * Make a GET request.
     *
     * Note: GET requests cannot contain bodies, and so any body provided in the RequestInit will be ignored.
     *
     * @param requestUrl The URL to make the request to
     * @param requestInit The settings for how the request should be made
     * @returns a {@link Response} object
     */
    async get(requestUrl, requestInit = {}) {
        const requestInitWithMethod = {
            ...requestInit,
            method: HttpMethod.GET,
        };
        return await this.request(requestUrl, requestInitWithMethod);
    }
    /**
     * Make a DELETE request.
     *
     * @param requestUrl The URL to make the request to
     * @param requestInit The settings for how the request should be made
     * @returns a {@link Response} object
     */
    async delete(requestUrl, requestInit = {}) {
        const requestInitWithMethod = {
            ...requestInit,
            method: HttpMethod.DELETE,
        };
        return await this.request(requestUrl, requestInitWithMethod);
    }
    /**
     * Make a POST request.
     *
     * @param requestUrl The URL to make the request to
     * @param requestInit The settings for how the request should be made
     * @returns a {@link Response} object
     */
    async post(requestUrl, requestInit = {}) {
        const requestInitWithMethod = {
            ...requestInit,
            method: HttpMethod.POST,
        };
        return await this.request(requestUrl, requestInitWithMethod);
    }
    /**
     * Make a PATCH request.
     *
     * @param requestUrl The URL to make the request to
     * @param requestInit The settings for how the request should be made
     * @returns a {@link Response} object
     */
    async patch(requestUrl, requestInit = {}) {
        const requestInitWithMethod = {
            ...requestInit,
            method: HttpMethod.PATCH,
        };
        return await this.request(requestUrl, requestInitWithMethod);
    }
    /**
     * Make a PUT request.
     *
     * @param requestUrl The URL to make the request to
     * @param requestInit The settings for how the request should be made
     * @returns a {@link Response} object
     */
    async put(requestUrl, requestInit = {}) {
        const requestInitWithMethod = {
            ...requestInit,
            method: HttpMethod.PUT,
        };
        return await this.request(requestUrl, requestInitWithMethod);
    }
    /**
     * Makes a raw fetch request.
     * All other methods such as get, post, patch, and request ultimately call this.
     * The normal get, post, patch, et all methods should be preferred.
     *
     * @param requestUrl The URL to make the request to
     * @param requestInit The {@link MethodSpecificRequestInit} settings for how the request should be made
     * @returns a {@link Response} object
     */
    async request(requestUrl, requestInit) {
        // make copy of requestInit to make sure the original is maintained
        const copyOfRequestInit = { ...requestInit };
        copyOfRequestInit.headers = this._getHeadersWithUserAgentMixedIn(copyOfRequestInit.headers);
        // Form the Request object.
        const request = new Request(requestUrl, copyOfRequestInit);
        // Use the Request object to make the request.
        const response = await fetch(request);
        return response;
    }
    /**
     * Create the {@link Headers} object to be used for the request.
     *
     * A {@link Headers} object is preferred by cross-fetch for the headers. It also allows for case-insensitive
     * lookups of header keys, which is convenient.
     *
     * @param headers The {@link HeadersInit} settings that will be used to produce the headers for the request
     * @returns The formed {@link Headers} object
     */
    _getHeadersWithUserAgentMixedIn(headers) {
        const userAgentUpdatedHeaders = new Headers(headers);
        if (this.userAgent) {
            userAgentUpdatedHeaders.set("User-Agent", this.userAgent);
        }
        return userAgentUpdatedHeaders;
    }
}
//# sourceMappingURL=HttpClient.js.map